Option Strict On

Imports System
Imports System.Data
Imports System.Collections
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.Design
#If NET_2_0 = 1 Then
Imports System.Web.UI.Design.WebControls ' DataboundControlDesigner definition
#End If
Imports System.ComponentModel
Imports System.ComponentModel.Design   'ComponentChangedEventArgs
Imports System.Diagnostics    ' Debug

' -------------------------- Databound Schedule Controls v1.6/1.9 -----------------------
'
' ------------------------------ www.rekenwonder.com ---------------------------------

Namespace rw

    ''' -----------------------------------------------------------------------------
    ''' Project	 : schedule
    ''' Class	 : rw.ScheduleDesigner
    '''
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' ScheduleDesigner is a virtual class that adds design-time support
    ''' Base class for ScheduleCalendarDesigner and ScheduleGeneralDesiger
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class ScheduleDesigner
#If Not NET_2_0 = 1 Then
        Inherits SimpleTemplatedControlDesigner
        Implements IDataSourceProvider
#Else
        Inherits DataBoundControlDesigner
        'Implements IDynamicControlDesigner
        'Implements IDataSourceProvider
#End If

#If Not NET_2_0 = 1 Then
        Protected dummyDataTable As DataTable
        Private designTimeDataTable As DataTable
#Else
        Private internalControl As BaseSchedule = Nothing
#End If

#If Not NET_2_0 = 1 Then

        Public Overrides ReadOnly Property Verbs() As DesignerVerbCollection
            Get
                Dim _verbs() As DesignerVerb = {New DesignerVerb("Property Builder...", _
                    New System.EventHandler(AddressOf OnPropertyBuilder))}
                Return New DesignerVerbCollection(_verbs)
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Get and set DataMember property from control
        ''' </summary>
        ''' <returns>String</returns>
        ''' -----------------------------------------------------------------------------
        Public Property DataMember() As String
            Get
                Dim control As BaseSchedule = CType(Me.Component, BaseSchedule)
                Return control.DataMember
            End Get
            Set(ByVal Value As String)
                Dim control As BaseSchedule = CType(Me.Component, BaseSchedule)
                control.DataMember = Value
            End Set
        End Property

        Public Property DataSource() As String
            Get
                Dim binding As DataBinding = DataBindings("DataSource")

                If Not (binding Is Nothing) Then
                    Return binding.Expression
                End If
                Return String.Empty
            End Get
            Set(ByVal Value As String)
                If Value Is Nothing OrElse Value.Length = 0 Then
                    DataBindings.Remove("DataSource")
                Else
                    Dim binding As DataBinding = DataBindings("DataSource")

                    If binding Is Nothing Then
                        binding = New DataBinding("DataSource", GetType(IEnumerable), Value)
                    Else
                        binding.Expression = Value
                    End If
                    DataBindings.Add(binding)
                End If

                OnDataSourceChanged()
                OnBindingsCollectionChanged("DataSource")
            End Set
        End Property

        Public Overrides ReadOnly Property DesignTimeHtmlRequiresLoadComplete() As Boolean
            Get
                ' If there is a data source, look it up in the container
                ' and require the document to be loaded completely.
                Return DataSource.Length <> 0
            End Get
        End Property
#End If

        Public Sub New() ' hide constructor in this abstract class
#If NET_2_0 = 1 Then
            ' Is this necessary? And if so, should it be here?
            ' According to a forum post, this line should add an option to the smart tag
            ' to edit templates, but it doesn't.
            SetViewFlags(ViewFlags.TemplateEditing, True)
#End If
        End Sub

#If NET_2_0 = 1 Then
        Public Overrides Sub Initialize(ByVal component As IComponent)
            ' Ensure that only a Schedule control can be created in this designer. 
            If Not TypeOf component Is BaseSchedule Then
                Throw New ArgumentException("The component is not a Schedule control.")
            End If
            internalControl = CType(component, BaseSchedule)
            MyBase.Initialize(component)
        End Sub ' Initialize
#End If

        Protected MustOverride Function GetDummyDataTable(ByVal minimumRows As Integer) As DataTable

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Shows a Property Builder dialog box
        ''' This method is overridden in ScheduleCalendarDesigner and ScheduleGeneralDesigner
        ''' to show the appropriate dialog box
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e"></param>
        ''' -----------------------------------------------------------------------------
        Public MustOverride Sub OnPropertyBuilder(ByVal sender As Object, ByVal e As EventArgs)

#If Not NET_2_0 = 1 Then
        Protected Function GetDesignTimeDataSource(ByVal minimumRows As Integer, ByRef dummyDataSource As Boolean) As IEnumerable
            dummyDataSource = False

            Dim selectedDataSource As IEnumerable = CType(Me, IDataSourceProvider).GetResolvedSelectedDataSource()
            Dim dataTable As DataTable = designTimeDataTable

            ' Use the data table corresponding to the selected data source if possible.
            If dataTable Is Nothing Then
                If Not (selectedDataSource Is Nothing) Then
                    designTimeDataTable = DesignTimeData.CreateSampleDataTable(selectedDataSource)
                    dataTable = designTimeDataTable
                End If

                If dataTable Is Nothing Then
                    ' Fall back on a dummy data source if a sample data-table cannot be created.
                    dataTable = GetDummyDataTable(minimumRows)
                    dummyDataSource = True
                End If
            End If

            Dim liveDataSource As IEnumerable = DesignTimeData.GetDesignTimeDataSource(dataTable, minimumRows)
            Return liveDataSource
        End Function
#Else
        Protected Overrides Function GetDesignTimeDataSource() As System.Collections.IEnumerable
            Return New DataView(GetDummyDataTable(5))
        End Function

#End If

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the HTML that is used to represent the control at design time..
        ''' </summary>
        ''' <returns>A string with the HTML</returns>
        ''' -----------------------------------------------------------------------------
        Public Overrides Function GetDesignTimeHtml() As String
#If Not NET_2_0 = 1 Then
            Dim internalControl As BaseSchedule = CType(Me.Component, BaseSchedule)
#End If
            Dim designTimeHTML As String = Nothing
            Dim hasATemplate As Boolean = Not Me.TemplatesAreEmpty()

            If hasATemplate Then
                Dim strCheckConfiguration As String = internalControl.CheckConfiguration()
                If (strCheckConfiguration <> "") Then
                    Return CreatePlaceHolderDesignTimeHtml(strCheckConfiguration)
                End If
#If Not NET_2_0 = 1 Then
                Dim dummyDataSource As Boolean
                Dim designTimeDataSource As IEnumerable = GetDesignTimeDataSource(5, dummyDataSource)
                Try
                    internalControl.DataSource = designTimeDataSource

                    designTimeHTML = MyBase.GetDesignTimeHtml()
                Finally
                    internalControl.DataSource = Nothing
                End Try
#Else
                Try
                    designTimeHTML = MyBase.GetDesignTimeHtml()
                Finally
                    'control.DataSource = Nothing
                End Try
#End If
            Else
                designTimeHTML = GetEmptyDesignTimeHtml()
            End If

            Return designTimeHTML
        End Function

        Protected Overrides Function GetErrorDesignTimeHtml(ByVal e As Exception) As String
            Debug.Fail(e.ToString())
            Return CreatePlaceHolderDesignTimeHtml("There was an error rendering the control.<br>Check to make sure all properties are valid.")
        End Function

#If Not NET_2_0 = 1 Then
        Private Shared validNames() As String = {"AlternatingItemStyle", "DataSource", _
            "DataMember", "RangeHeaderStyle", "TitleStyle", "BackgroundStyle", _
            "ItemStyle"}
#Else
        Private Shared validNames() As String = {"AlternatingItemStyle", "DataSourceID", _
            "DataMember", "RangeHeaderStyle", "TitleStyle", "BackgroundStyle", _
            "ItemStyle"}
#End If

        Public Overrides Sub OnComponentChanged(ByVal sender As Object, ByVal e As ComponentChangedEventArgs)
            If Not (e.Member Is Nothing) Then
                Dim memberName As String = e.Member.Name
                Dim current As String
                For Each current In validNames
                    If memberName = current Then
#If Not NET_2_0 = 1 Then
                        OnStylesChanged()
#End If
                        Exit For
                    End If
                Next current
#If Not NET_2_0 = 1 Then
                Dim DataMemberNames() As String = {"DataSource", "DataMember", "TitleField", "DateField", _
                    "DataRangeStartField", "DataRangeEndField", "StartTimeField", "EndTimeField"}
#Else
                Dim DataMemberNames() As String = {"DataSourceID", "DataMember", "TitleField", "DateField", _
                    "DataRangeStartField", "DataRangeEndField", "StartTimeField", "EndTimeField"}
#End If
                If Array.IndexOf(DataMemberNames, memberName) > -1 Then
#If Not NET_2_0 = 1 Then
                    OnDataSourceChanged()
#Else
                    OnDataSourceChanged(True)
#End If
                End If
            End If
            MyBase.OnComponentChanged(sender, e)

        End Sub 'OnComponentChanged

#If Not NET_2_0 = 1 Then
        Protected Overrides ReadOnly Property EditTemplateInstructions() As String
            Get
                Dim _text As String
                If CanEnterTemplateMode Then
                    _text = "Right-click and choose a set of templates to edit their content.<br>The ItemTemplate is required."
                Else
                    _text = "Switch to HTML view to edit the control's templates.<br>The ItemTemplate is required."
                End If
                Return _text
            End Get
        End Property

        Public Overrides Function GetTemplateContainerDataSource(ByVal templateName As String) As IEnumerable
            Return GetResolvedSelectedDataSource()
        End Function

        Public Overrides Function GetTemplateContainerDataItemProperty(ByVal templateName As String) As String
            Return "DataItem"
        End Function

        Protected Overridable Sub OnDataSourceChanged()
            designTimeDataTable = Nothing
        End Sub

        Protected Overrides Sub PreFilterProperties(ByVal properties As IDictionary)
            ' This code is courtesy of Shaun Wilde on CodeProject
            ' http://www.codeproject.com/aspnet/webcontrolsdatabinding.asp
            Dim strDataSourceProperty As String = "DataSource"
            MyBase.PreFilterProperties(properties)
            Dim prop As PropertyDescriptor = CType(properties(strDataSourceProperty), PropertyDescriptor)
            If (Not prop Is Nothing) Then
                Dim runtimeAttributes As System.ComponentModel.AttributeCollection = prop.Attributes
                ' make a copy of the original attributes but make room for one extra
                ' attribute ie the TypeConverter attribute
                Dim attrs(runtimeAttributes.Count) As Attribute
                runtimeAttributes.CopyTo(attrs, 0)
                attrs(runtimeAttributes.Count) = New TypeConverterAttribute(GetType(DataSourceConverter))
                prop = TypeDescriptor.CreateProperty(Me.GetType(), strDataSourceProperty, GetType(String), attrs)
                properties(strDataSourceProperty) = prop
            End If
        End Sub

        Function GetResolvedSelectedDataSource() As IEnumerable Implements IDataSourceProvider.GetResolvedSelectedDataSource
            Return CType(GetSelectedDataSource(), IEnumerable)
        End Function

        Function GetSelectedDataSource() As Object Implements IDataSourceProvider.GetSelectedDataSource

            Dim selectedDataSource As Object = Nothing
            Dim dataSource2 As String = Nothing

            Dim binding As DataBinding = DataBindings("DataSource")
            If Not (binding Is Nothing) Then
                dataSource2 = binding.Expression
            End If

            If Not (dataSource2 = Nothing) Then
                Dim componentSite As ISite = Component.Site
                If Not (componentSite Is Nothing) Then
                    Dim container As IContainer = CType(componentSite.GetService(GetType(IContainer)), IContainer)

                    If Not (container Is Nothing) Then
                        Dim comp As IComponent = container.Components(dataSource2)
                        If TypeOf comp Is IEnumerable Then
                            selectedDataSource = comp
                        End If
                    End If
                End If
            End If

            Return selectedDataSource
        End Function  'GetSelectedDataSource

#Else ' NET_2_0 = 1
        Private Const templateGroupName1 As String = "Item Templates"
        Private Const templateDefinitionName1 As String = "ItemTemplate"
        Private Const templateDefinitionName2 As String = "AlternatingItemTemplate"
        Private Const templateGroupName2 As String = "Header Templates"
        Private Const templateDefinitionName3 As String = "TitleTemplate"
        Private Const templateDefinitionName4 As String = "RangeHeaderTemplate"
        Private internalGroup1 As TemplateGroup = Nothing
        Private internalGroup2 As TemplateGroup = Nothing

        Private lists As DesignerActionListCollection

        Public Overrides ReadOnly Property ActionLists() As DesignerActionListCollection
            Get
                If lists Is Nothing Then
                    lists = MyBase.ActionLists
                    lists.Add(New ScheduleDesignerActionList(Me.Component, Me))
                End If
                Return lists
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Template definitions for the Schedule control.
        ''' NOTE: this is not working!!!
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Overrides ReadOnly Property TemplateGroups() As TemplateGroupCollection
            Get
                ' Start with the groups defined by the base designer class.
                Dim groups As TemplateGroupCollection = MyBase.TemplateGroups

                If internalGroup1 Is Nothing Then

                    ' Define a new group with two template definitions.
                    internalGroup1 = New TemplateGroup(templateGroupName1, internalControl.ControlStyle)

                    Dim templateDef1 As TemplateDefinition = New TemplateDefinition(Me, _
                        templateDefinitionName1, internalControl, _
                        templateDefinitionName1, internalControl.ControlStyle)

                    Dim templateDef2 As TemplateDefinition = New TemplateDefinition(Me, _
                        templateDefinitionName2, internalControl, _
                        templateDefinitionName2, internalControl.ControlStyle)

                    internalGroup1.AddTemplateDefinition(templateDef1)
                    internalGroup1.AddTemplateDefinition(templateDef2)

                    ' Define a new group with two template definitions.
                    internalGroup2 = New TemplateGroup(templateGroupName2, internalControl.ControlStyle)

                    Dim templateDef3 As TemplateDefinition = New TemplateDefinition(Me, _
                        templateDefinitionName3, internalControl, _
                        templateDefinitionName3, internalControl.ControlStyle)

                    Dim templateDef4 As TemplateDefinition = New TemplateDefinition(Me, _
                        templateDefinitionName4, internalControl, _
                        templateDefinitionName4, internalControl.ControlStyle)

                    internalGroup2.AddTemplateDefinition(templateDef3)
                    internalGroup2.AddTemplateDefinition(templateDef4)
                End If

                ' Add the new template group to the collection.
                groups.Add(internalGroup1)
                groups.Add(internalGroup2)

                Return groups
            End Get
        End Property

        'Probably handled by DataBoundControlDesigner
        Public Overrides ReadOnly Property DesignTimeHtmlRequiresLoadComplete() As Boolean
            Get
                ' If there is a data source, look it up in the container
                ' and require the document to be loaded completely.
                Return DataSource.Length <> 0
            End Get
        End Property

        ' Create a read-only property that determines whether the control
        ' is using templates at design time.      
        Protected ReadOnly Property TemplatesExist() As Boolean
            Get
                Return True
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Create a read-only property that determines whether the control
        ''' is using templates at design time.
        ''' </summary>
        ''' <returns>True if Templates are empty</returns>
        ''' -----------------------------------------------------------------------------
        Protected ReadOnly Property TemplatesAreEmpty() As Boolean
            Get
                Return TemplateGroups.Count = 0
            End Get
        End Property

#End If
    End Class

End Namespace

