Option Strict On

Imports System
Imports System.Text
Imports System.Data
Imports System.IO
Imports System.Collections
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.Design
Imports Microsoft.VisualBasic   ' needed for IIf definition
Imports Microsoft.VisualBasic.DateAndTime
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Diagnostics    ' Debug
Imports System.Reflection    ' PropertyInfo definition

' -------------------------- Databound Schedule Controls v1.6/1.9 -----------------------
'
' ------------------------------ www.rekenwonder.com ---------------------------------

Namespace rw
    ''' -----------------------------------------------------------------------------
    ''' Project	 : schedule
    ''' Class	 : rw.ScheduleGeneralDesigner
    '''
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' ScheduleGeneralDesigner adds design-time support for the ScheduleGeneral control
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Class ScheduleGeneralDesigner
        Inherits ScheduleDesigner

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Provides a dummy DataTable object, allowing the ScheduleGeneral control to
        ''' show some sample data in the designer.
        ''' </summary>
        ''' <param name="minimumRows"></param>
        ''' <returns>A dummy DataTable object</returns>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Function GetDummyDataTable(ByVal minimumRows As Integer) As DataTable
            'Create a new dummy table every time this function gets called.
            'The alternative is to raise OnDataSourceChanged every time DateField, TimeFieldsContainDate etc. change
            Dim control As ScheduleGeneral = CType(Me.Component, ScheduleGeneral)
            Dim dummyDataTable As DataTable
            dummyDataTable = New DataTable
            dummyDataTable.Columns.Add(New DataColumn(control.TitleField, System.Type.GetType("System.String")))
            dummyDataTable.Columns.Add(New DataColumn(control.DataRangeStartField, System.Type.GetType("System.DateTime")))
            If (control.DataRangeEndField <> control.DataRangeStartField) Then
                dummyDataTable.Columns.Add(New DataColumn(control.DataRangeEndField, System.Type.GetType("System.DateTime")))
            End If
            dummyDataTable.Columns.Add(New DataColumn(control.ItemStyleField, System.Type.GetType("System.String")))

            ' create some dummy rows
            Dim d As Date = Today()
            Dim i As Integer
            For i = 0 To minimumRows - 1
                Dim row As DataRow = dummyDataTable.NewRow()
                d = d.AddDays(1)   ' create rows on successive days
                row.Item(0) = "Title " & (i + 1)
                row.Item(1) = New DateTime(d.Year, d.Month, d.Day, 9 + i Mod 2, 0, 0)
                If (control.DataRangeEndField <> control.DataRangeStartField) Then
                    row.Item(2) = New DateTime(d.Year, d.Month, d.Day, 12 + i Mod 2, 0, 0)
                End If
                dummyDataTable.Rows.Add(row)
            Next

            For i = 0 To minimumRows - 1
                Dim row As DataRow = dummyDataTable.NewRow()
                d = d.AddDays(1)   ' create rows on successive days
                row.Item(0) = "Title " & (i + 1)
                row.Item(1) = New DateTime(d.Year, d.Month, d.Day, 9 + i Mod 2, 0, 0)
                If (control.DataRangeEndField <> control.DataRangeStartField) Then
                    row.Item(2) = New DateTime(d.Year, d.Month, d.Day, 12 + i Mod 2, 0, 0)
                    row.Item(3) = "dummystyle"
                Else
                    row.Item(2) = "dummystyle"
                End If
                dummyDataTable.Rows.Add(row)
            Next

            Return dummyDataTable
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Shows a Property Builder dialog box
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e"></param>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub OnPropertyBuilder(ByVal sender As Object, ByVal e As EventArgs)
            Dim control As ScheduleGeneral = CType(Me.Component, ScheduleGeneral)
            Dim gpb As New GeneralPropertyBuilder
            gpb.tbTitleField.Text = control.TitleField
            gpb.tbDataRangeStartField.Text = control.DataRangeStartField
            gpb.tbDataRangeEndField.Text = control.DataRangeEndField
            gpb.tbItemStyleField.Text = control.ItemStyleField
            gpb.tbTitleFormatString.Text = control.TitleDataFormatString
            gpb.tbRangeDataFormatString.Text = control.RangeDataFormatString
            gpb.tbDateHeaderFormatString.Text = control.DateHeaderDataFormatString
            gpb.cbAutoSortTitles.Checked = control.AutoSortTitles
            gpb.cbSeparateDateHeader.Checked = control.SeparateDateHeader
            gpb.tbDateHeaderFormatString.Enabled = control.SeparateDateHeader
            gpb.cbIncludeEndValue.Checked = control.IncludeEndValue
            gpb.cbShowValueMarks.Checked = control.ShowValueMarks
            gpb.rbHorizontal.Checked = (control.Layout = LayoutEnum.Horizontal)
            gpb.rbVertical.Checked = (control.Layout = LayoutEnum.Vertical)
            gpb.cbFullTimeScale.Checked = control.FullTimeScale
            gpb.tbStartTimeScale.Text = control.StartOfTimeScale.ToString()
            gpb.tbEndTimeScale.Text = control.EndOfTimeScale.ToString()
            gpb.tbInterval.Text = control.TimeScaleInterval.ToString()
            gpb.tbStartTimeScale.Enabled = control.FullTimeScale
            gpb.tbEndTimeScale.Enabled = control.FullTimeScale
            gpb.tbInterval.Enabled = control.FullTimeScale

            If (gpb.ShowDialog() = Windows.Forms.DialogResult.OK) Then
                control.TitleField = gpb.tbTitleField.Text
                control.DataRangeStartField = gpb.tbDataRangeStartField.Text
                control.DataRangeEndField = gpb.tbDataRangeEndField.Text
                control.ItemStyleField = gpb.tbItemStyleField.Text
                control.TitleDataFormatString = gpb.tbTitleFormatString.Text
                control.RangeDataFormatString = gpb.tbRangeDataFormatString.Text
                control.DateHeaderDataFormatString = gpb.tbDateHeaderFormatString.Text
                control.AutoSortTitles = gpb.cbAutoSortTitles.Checked
                control.SeparateDateHeader = gpb.cbSeparateDateHeader.Checked
                control.IncludeEndValue = gpb.cbIncludeEndValue.Checked
                control.ShowValueMarks = gpb.cbShowValueMarks.Checked
                control.Layout = CType(IIf(gpb.rbHorizontal.Checked, LayoutEnum.Horizontal, LayoutEnum.Vertical), LayoutEnum)
                control.FullTimeScale = gpb.cbFullTimeScale.Checked
                control.StartOfTimeScale = TimeSpan.Parse(gpb.tbStartTimeScale.Text)
                control.EndOfTimeScale = TimeSpan.Parse(gpb.tbEndTimeScale.Text)
                control.TimeScaleInterval = Int32.Parse(gpb.tbInterval.Text)
                RaiseComponentChanged(Nothing, Nothing, Nothing)
                UpdateDesignTimeHtml()
            End If
        End Sub

    End Class

End Namespace

